/** 
* Referenced from FlyControls.js by James Baicoianu / http://www.baicoianu.com/
* https://github.com/mrdoob/three.js/blob/master/examples/js/controls/FlyControls.js

* also referenced FirstPersonControls.js 
https://github.com/IceCreamYou/Nemesis/blob/master/Three.FirstPersonControls.js
*/

var firstPersonCamera = (function (V) {
  "use strict";
  // constants
  const UP_VECTOR = new THREE.Vector3(0,1,0);
  const INITIAL_TARGET = new THREE.Vector3(0,V.PLAYER_HEIGHT,0);
  const INIT_X_OFFSET = -V.ROOM_WIDTH/2 + V.TABLE_ROW_WIDTH + 50;
  const X_BOUNDARY = V.ROOM_WIDTH/2;
  const Z_BOUNDARY = V.ROOM_LENGTH/2;
  const Y_BOUNDARY = V.PLAYER_HEIGHT;
  const Y_PADDING = 5;
  
  var camera = V.camera;
  var targetPosition = INITIAL_TARGET.clone();
  var lat = 0;
  var lon = 0;
  var driftlat = 0;
  var driftlon = 0;
  var rotationMultiplier = 0.0095;
  var halfWidth  = document.body.clientWidth / 2;
  var halfHeight = document.body.clientHeight / 2;  
  
  // SETUP CAMERA
  function initCamera(){
    camera.position.set(INIT_X_OFFSET,V.PLAYER_HEIGHT,0);
    camera.lookAt(targetPosition);

    view.scene.add(camera);
    document.addEventListener("mousemove", onMouseMove, false);
	
  }

  function onMouseMove (event){
    lon += event.mozMovementX * rotationMultiplier;
    lat -= event.mozMovementY * rotationMultiplier;
  }
	
  function checkBoundaries(camera, lockedPosition, crouching){
      if((camera.position.x > X_BOUNDARY) || (camera.position.x < -X_BOUNDARY)){
          camera.position.x = lockedPosition.x;
      }

      if((camera.position.z > Z_BOUNDARY) || (camera.position.z < -Z_BOUNDARY + V.FRIDGE_BOUNDARY)){
          camera.position.z = lockedPosition.z;
      }
      //Top Right blocker
      if((camera.position.x > X_BOUNDARY - V.BLOCKER_WIDTH) && (camera.position.z > Z_BOUNDARY - V.BLOCKER_LENGTH)){
          camera.position.x = lockedPosition.x;
          camera.position.z = lockedPosition.z;
      }
      //Bottom Right blocker
      if((camera.position.x < -X_BOUNDARY + V.TABLE_ROW_WIDTH) && (camera.position.z > Z_BOUNDARY - V.TABLE_ROW_LENGTH)){
          camera.position.x = lockedPosition.x;
          camera.position.z = lockedPosition.z;
      }
      if (crouching) {
        if(camera.position.y <= Y_PADDING) {
          camera.position.y = Y_PADDING;
        } else if (camera.position.y >= V.PLAYER_CROUCH_HEIGHT) {
          camera.position.y = V.PLAYER_CROUCH_HEIGHT;
        }
      } else {
        if(camera.position.y <= V.PLAYER_CROUCH_HEIGHT) {
          camera.position.y = V.PLAYER_CROUCH_HEIGHT;
        } else if (camera.position.y >= V.PLAYER_HEIGHT + Y_PADDING) {
          camera.position.y = V.PLAYER_HEIGHT;
        }
      }

  }
	
  function update(dx, dy, dz, crouching){
      var lockedPosition = camera.position.clone();
      camera.position.y = lockedPosition.y;
      camera.translateX(dx);
      camera.translateZ(dz);
      camera.translateY(dy);
    
      checkBoundaries(camera, lockedPosition, crouching);

      lat = Math.max(-85, Math.min(85, lat));
      var phi   = ( 90 - lat ) * Math.PI / 180;
      var theta = lon * Math.PI / 180;
      targetPosition.x = camera.position.x + 100 * Math.sin(phi) * Math.cos(theta);
      targetPosition.y = camera.position.y + 100 * Math.cos(phi);
      targetPosition.z = camera.position.z + 100 * Math.sin(phi) * Math.sin(theta);
      camera.up = V.Y_AXIS;
      camera.lookAt(targetPosition);

  }
  
  //Debugging tools
  function faceCameraAt(facing){
	  switch(facing){
		case "door":
			camera.position.set(INIT_X_OFFSET,V.PLAYER_HEIGHT,0);
			targetPosition = INITIAL_TARGET;
			camera.lookAt(INITIAL_TARGET);
			break;
	  }
	  camera.updateProjectionMatrix();
  }
  
  return {
    camera: camera,
    initCamera: initCamera,
    update: update,
	faceCameraAt: faceCameraAt
  };
})(V);
